@@ -29,16 +29,27 @@ def getValues(self, rows):
 
 vd.aggregators = collections.OrderedDict()  # [aggname] -> annotated func, or list of same
 
-Column.init('aggregators', list)
+Column.init('aggstr', str, copy=True)
 
-def aggstr_get(col):
+def aggregators_get(col):
     'A space-separated names of aggregators on this column.'
-    return ' '.join(aggr.name for aggr in col.aggregators)
+    return list(vd.aggregators[k] for k in (col.aggstr or '').split())
 
-def aggstr_set(col, v):
-    col.aggregators = list(vd.aggregators[k] for k in (v or '').split())
+def aggregators_set(col, aggs):
+    if isinstance(aggs, str):
+        newaggs = []
+        for agg in aggs.split():
+            if agg not in vd.aggregators:
+                vd.fail(f'unknown aggregator {agg}')
+            newaggs.append(agg)
+    elif aggs is None:
+        newaggs = ''
+    else:
+        newaggs = [agg.name for agg in aggs]
 
-Column.aggstr = property(aggstr_get, aggstr_set)
+    col.aggstr = ' '.join(newaggs)
+
+Column.aggregators = property(aggregators_get, aggregators_set)
 
 
 class Aggregator:
@@ -54,7 +65,7 @@ class Aggregator:
 
 _defaggr = Aggregator
 
-@VisiData.global_api
+@VisiData.api
 def aggregator(vd, name, func, helpstr='', *args, type=None):
     'Define simple aggregator *name* that calls ``func(values, *args)`` to aggregate *values*.  Use *type* to force the default type of the aggregated column.'
     def _func(col, rows):  # wrap builtins so they can have a .type
@@ -67,6 +78,7 @@ def aggregator(vd, name, func, helpstr='', *args, type=None):
             return e
 
     vd.aggregators[name] = _defaggr(name, type, _func, helpstr)
+    vd.addGlobals({name: func})
 
 ## specific aggregator implementations
 
@@ -123,13 +135,22 @@ vd.aggregator('stdev', stdev, 'standard deviation of values', type=float)
 vd.aggregators['q3'] = quantiles(3, 'tertiles (33/66th pctile)')
 vd.aggregators['q4'] = quantiles(4, 'quartiles (25/50/75th pctile)')
 vd.aggregators['q5'] = quantiles(5, 'quintiles (20/40/60/80th pctiles)')
-vd.aggregators['q10'] = quantiles(10, 'deciles (10/20/30/40/50/60/70/80/80th pctiles)')
+vd.aggregators['q10'] = quantiles(10, 'deciles (10/20/30/40/50/60/70/80/90th pctiles)')
+
+# since bb29b6e, a record of every aggregator
+# is needed in vd.aggregators
+for pct in (10, 20, 25, 30, 33, 40, 50, 60, 67, 70, 75, 80, 90, 95, 99):
+    vd.aggregators[f'p{pct}'] = percentile(pct, f'{pct}th percentile')
 
 # returns keys of the row with the max value
 vd.aggregators['keymax'] = _defaggr('keymax', anytype, lambda col, rows: col.sheet.rowkey(max(col.getValueRows(rows))[1]), 'key of the maximum value')
 
 
-ColumnsSheet.columns += [ColumnAttr('aggregators','aggstr')]
+ColumnsSheet.columns += [
+    Column('aggregators',
+           getter=lambda c,r:r.aggstr,
+           setter=lambda c,r,v:setattr(r, 'aggregators', v))
+]
 
 @Sheet.api
 def addAggregators(sheet, cols, aggrnames):
@@ -152,12 +173,14 @@ def aggname(col, agg):
 
 @Column.api
 @asyncthread
-def show_aggregate(col, agg, rows):
+def memo_aggregate(col, agg, rows):
     'Show aggregated value in status, and add to memory.'
     aggval = agg(col, rows)
     typedval = wrapply(agg.type or col.type, aggval)
     dispval = col.format(typedval)
-    vd.status(dispval)
+    k = col.name+'_'+agg.name
+    vd.status(f'{k}={dispval}')
+    vd.memory[k] = typedval
 
 
 @VisiData.property
@@ -166,9 +189,7 @@ def aggregator_choices(vd):
        {'key': agg, 'desc': v[0].helpstr if isinstance(v, list) else v.helpstr} for agg, v in vd.aggregators.items()
     ]
 
-vd.addGlobals(globals())
 
-
-Sheet.addCommand('+', 'aggregate-col', 'addAggregators([cursorCol], chooseMany(aggregator_choices))', 'add aggregator to current column')
-Sheet.addCommand('z+', 'show-aggregate', 'for agg in chooseMany(aggregator_choices): cursorCol.show_aggregate(aggregators[agg], selectedRows or rows)', 'display result of aggregator over values in selected rows for current column')
+Sheet.addCommand('+', 'aggregate-col', 'addAggregators([cursorCol], chooseMany(aggregator_choices))', 'Add aggregator to current column')
+Sheet.addCommand('z+', 'memo-aggregate', 'for agg in chooseMany(aggregator_choices): cursorCol.memo_aggregate(aggregators[agg], selectedRows or rows)', 'memo result of aggregator over values in selected rows for current column')
 ColumnsSheet.addCommand('g+', 'aggregate-cols', 'addAggregators(selectedRows or source[0].nonKeyVisibleCols, chooseMany(aggregator_choices))', 'add aggregators to selected source columns')
