@@ -3,52 +3,70 @@ import threading
 from visidata import *
 import visidata
 
-option('replay_wait', 0.0, 'time to wait between replayed commands, in seconds', sheettype=None)
-theme('disp_replay_play', '▶', 'status indicator for active replay')
-theme('disp_replay_pause', '‖', 'status indicator for paused replay')
-theme('color_status_replay', 'green', 'color of replay status indicator')
-option('replay_movement', False, 'insert movements during replay', sheettype=None)
-option('visidata_dir', '~/.visidata/', 'directory to load and store additional files', sheettype=None)
+vd.option('replay_wait', 0.0, 'time to wait between replayed commands, in seconds', sheettype=None)
+vd.option('disp_replay_play', '▶', 'status indicator for active replay')
+vd.option('disp_replay_pause', '‖', 'status indicator for paused replay')
+vd.option('color_status_replay', 'green', 'color of replay status indicator')
+vd.option('replay_movement', False, 'insert movements during replay', sheettype=None)
 
 # prefixes which should not be logged
 nonLogged = '''forget exec-longname undo redo quit
 show error errors statuses options threads jump
-replay cancel save-cmdlog macro
-go- search scroll prev next page start end zoom resize visibility
+replay cancel save-cmdlog macro cmdlog-sheet menu repeat
+go- search scroll prev next page start end zoom resize visibility sidebar
 mouse suspend redraw no-op help syscopy sysopen profile toggle'''.split()
 
-option('rowkey_prefix', 'キ', 'string prefix for rowkey in the cmdlog', sheettype=None)
-option('cmdlog_histfile', '', 'file to autorecord each cmdlog action to', sheettype=None)
+vd.option('rowkey_prefix', 'キ', 'string prefix for rowkey in the cmdlog', sheettype=None)
+vd.option('cmdlog_histfile', '', 'file to autorecord each cmdlog action to', sheettype=None)
 
 vd.activeCommand = UNLOADED
 
-def open_vd(p):
+@VisiData.api
+def open_vd(vd, p):
     return CommandLog(p.name, source=p, precious=True)
 
-def open_vdj(p):
+@VisiData.api
+def open_vdj(vd, p):
     return CommandLogJsonl(p.name, source=p, precious=True)
 
 VisiData.save_vd = VisiData.save_tsv
-VisiData.save_vdj = VisiData.save_jsonl
 
 
-def checkVersion(desired_version):
+@VisiData.api
+def save_vdj(vd, p, *vsheets):
+    with p.open_text(mode='w', encoding=vsheets[0].options.encoding) as fp:
+        fp.write("#!vd -p\n")
+        for vs in vsheets:
+            vs.write_jsonl(fp)
+
+
+@VisiData.api
+def checkVersion(vd, desired_version):
     if desired_version != visidata.__version_info__:
         vd.fail("version %s required" % desired_version)
 
-def fnSuffix(prefix):
+@VisiData.api
+def fnSuffix(vd, prefix):
     i = 0
-    fn = prefix + '.vd'
+    fn = prefix + '.vdj'
     while Path(fn).exists():
         i += 1
-        fn = f'{prefix}-{i}.vd'
+        fn = f'{prefix}-{i}.vdj'
 
     return fn
 
+@BaseSheet.api
 def inputLongname(sheet):
     longnames = set(k for (k, obj), v in vd.commands.iter(sheet))
     return vd.input("command name: ", completer=CompleteKey(sorted(longnames)), type='longname')
 
+@BaseSheet.api
+def exec_longname(sheet, longname):
+    if not sheet.getCommand(longname):
+        vd.warning(f'no command {longname}')
+        return
+    sheet.execCommand(longname)
+
 def indexMatch(L, func):
     'returns the smallest i for which func(L[i]) is true'
     for i, x in enumerate(L):
@@ -58,7 +76,8 @@ def indexMatch(L, func):
 def keystr(k):
     return  options.rowkey_prefix+','.join(map(str, k))
 
-def isLoggableCommand(longname):
+@VisiData.api
+def isLoggableCommand(vd, longname):
     for n in nonLogged:
         if longname.startswith(n):
             return False
@@ -97,14 +116,14 @@ def getRowIndexFromStr(vs, rowstr):
         return None
 
 @Sheet.api
-def moveToCol(vs, colstr):
-    'Move cursor to column given by *colstr*, which can be either the column number or column name.'
-    try:
-        vcolidx = int(colstr)
-    except ValueError:
-        vcolidx = indexMatch(vs.visibleCols, lambda c,name=colstr: name == c.name)
+def moveToCol(vs, col):
+    'Move cursor to column given by *col*, which can be either the column number or column name.'
+    if isinstance(col, str):
+        vcolidx = indexMatch(vs.visibleCols, lambda c,name=col: name == c.name)
+    elif isinstance(col, int):
+        vcolidx = col
 
-    if vcolidx is None:
+    if vcolidx is None or vcolidx >= vs.nVisibleCols:
         return False
 
     if vs.options.replay_movement:
@@ -117,8 +136,23 @@ def moveToCol(vs, colstr):
 
     return True
 
+
+@BaseSheet.api
+def commandCursor(sheet, execstr):
+    'Return (col, row) of cursor suitable for cmdlog replay of execstr.'
+    colname, rowname = '', ''
+    contains = lambda s, *substrs: any((a in s) for a in substrs)
+    if contains(execstr, 'cursorTypedValue', 'cursorDisplay', 'cursorValue', 'cursorCell', 'cursorRow') and sheet.nRows > 0:
+        k = sheet.rowkey(sheet.cursorRow)
+        rowname = keystr(k) if k else sheet.cursorRowIndex
+
+    if contains(execstr, 'cursorTypedValue', 'cursorDisplay', 'cursorValue', 'cursorCell', 'cursorCol', 'cursorVisibleCol', 'ColumnAtCursor'):
+        colname = sheet.cursorCol.name or sheet.visibleCols.index(sheet.cursorCol)
+    return colname, rowname
+
+
 # rowdef: namedlist (like TsvSheet)
-class _CommandLog:
+class CommandLogBase:
     'Log of commands for current session.'
     rowtype = 'logged commands'
     precious = False
@@ -140,40 +174,23 @@ class _CommandLog:
         return self._rowtype(**fields)
 
     def beforeExecHook(self, sheet, cmd, args, keystrokes):
-        if not isLoggableCommand(cmd.longname):
-            return
-
         if vd.activeCommand:
             self.afterExecSheet(sheet, False, '')
 
         colname, rowname, sheetname = '', '', None
         if sheet and not (cmd.longname.startswith('open-') and not cmd.longname in ('open-row', 'open-cell')):
-            sheetname = sheet
+            sheetname = sheet.name
+
+            colname, rowname = sheet.commandCursor(cmd.execstr)
 
             contains = lambda s, *substrs: any((a in s) for a in substrs)
-            if contains(cmd.execstr, 'cursorTypedValue', 'cursorDisplay', 'cursorValue', 'cursorCell', 'cursorRow') and sheet.nRows > 0:
-                k = sheet.rowkey(sheet.cursorRow)
-                rowname = keystr(k) if k else sheet.cursorRowIndex
-
-            if contains(cmd.execstr, 'cursorTypedValue', 'cursorDisplay', 'cursorValue', 'cursorCell', 'cursorCol', 'cursorVisibleCol'):
-                colname = sheet.cursorCol.name or sheet.visibleCols.index(sheet.cursorCol)
-
-            if contains(cmd.execstr, 'plotterCursorBox'):
-                bb = sheet.cursorBox
-                colname = '%s %s' % (sheet.formatX(bb.xmin), sheet.formatX(bb.xmax))
-                rowname = '%s %s' % (sheet.formatY(bb.ymin), sheet.formatY(bb.ymax))
-            elif contains(cmd.execstr, 'plotterVisibleBox'):
-                bb = sheet.visibleBox
-                colname = '%s %s' % (sheet.formatX(bb.xmin), sheet.formatX(bb.xmax))
-                rowname = '%s %s' % (sheet.formatY(bb.ymin), sheet.formatY(bb.ymax))
-
             if contains(cmd.execstr, 'pasteFromClipboard'):
-                args = clipboard().paste().strip()
+                args = vd.sysclip_value().strip()
 
 
         comment = vd.currentReplayRow.comment if vd.currentReplayRow else cmd.helpstr
         vd.activeCommand = self.newRow(sheet=sheetname,
-                                            col=str(colname),
+                                            col=colname,
                                             row=str(rowname),
                                             keystrokes=keystrokes,
                                             input=args,
@@ -189,30 +206,42 @@ class _CommandLog:
         if err:
             vd.activeCommand[-1] += ' [%s]' % err
 
-        # remove user-aborted commands and simple movements
-        if not escaped and isLoggableCommand(vd.activeCommand.longname):
-            if isLoggableSheet(sheet):      # don't record actions on global cmdlog or other internal sheets
-                    self.addRow(vd.activeCommand)  # add to global cmdlog
+        if escaped:
+            vd.activeCommand = None
+            return
+
+        # remove user-aborted commands and simple movements (unless first command on the sheet, which created the sheet)
+        if not sheet.cmdlog.rows or vd.isLoggableCommand(vd.activeCommand.longname):
+            if isLoggableSheet(sheet):      # don't record actions from cmdlog or other internal sheets on global cmdlog
+                self.addRow(vd.activeCommand)  # add to global cmdlog
             sheet.cmdlog_sheet.addRow(vd.activeCommand)  # add to sheet-specific cmdlog
             if options.cmdlog_histfile:
+                name = date().strftime(options.cmdlog_histfile)
+                p = Path(name)
+                if not p.is_absolute():
+                    p = Path(options.visidata_dir)/f'{name}.jsonl'
                 if not getattr(vd, 'sessionlog', None):
-                    vd.sessionlog = vd.loadInternalSheet(CommandLog, Path(date().strftime(options.cmdlog_histfile)))
-                append_tsv_row(vd.sessionlog, vd.activeCommand)
+                    vd.sessionlog = vd.loadInternalSheet(CommandLog, p)
+                vd.sessionlog.append_tsv_row(vd.activeCommand)
 
         vd.activeCommand = None
 
     def openHook(self, vs, src):
-        r = self.newRow(keystrokes='o', input=src, longname='open-file')
+        while isinstance(src, BaseSheet):
+            src = src.source
+        r = self.newRow(keystrokes='o', input=str(src), longname='open-file')
         vs.cmdlog_sheet.addRow(r)
         self.addRow(r)
 
-class CommandLog(_CommandLog, VisiDataMetaSheet):
+class CommandLog(CommandLogBase, VisiDataMetaSheet):
     pass
 
-class CommandLogJsonl(_CommandLog, JsonLinesSheet):
+class CommandLogJsonl(CommandLogBase, JsonLinesSheet):
 
-    def newRow(self):
-        return JsonLinesSheet.newRow(self)
+    filetype = 'vdj'
+
+    def newRow(self, **fields):
+        return AttrDict(JsonLinesSheet.newRow(self, **fields))
 
     def iterload(self):
         for r in JsonLinesSheet.iterload(self):
@@ -257,11 +286,11 @@ def replay_cancel(vd):
 
 @VisiData.api
 def moveToReplayContext(vd, r, vs):
-        'set the sheet/row/col to the values in the replay row.  return sheet'
-        if r.row:
+        'set the sheet/row/col to the values in the replay row'
+        if r.row not in [None, '']:
             vs.moveToRow(r.row) or vd.error('no "%s" row' % r.row)
 
-        if r.col:
+        if r.col not in [None, '']:
             vs.moveToCol(r.col) or vd.error('no "%s" column' % r.col)
 
 
@@ -271,11 +300,6 @@ def delay(vd, factor=1):
         acquired = vd.semaphore.acquire(timeout=options.replay_wait*factor if not vd.paused else None)
         return acquired or not vd.paused
 
-@VisiData.property
-def activeSheet(vd):
-    'Return top sheet on sheets stack, or cmdlog sheets stack empty.'
-    return vd.sheets[0] if vd.sheets else vd.cmdlog
-
 
 @VisiData.api
 def replayOne(vd, r):
@@ -303,10 +327,11 @@ def replayOne(vd, r):
                 vd.exceptionCaught(e)
                 escaped = True
         else:
+            vs = vs or vd.activeSheet
             if vs:
                 vd.push(vs)
             else:
-                vs = vd.activeSheet
+                vs = vd.cmdlog
 
             vd.moveToReplayContext(r, vs)
 
@@ -349,7 +374,8 @@ def replay_sync(vd, cmdlog, live=False):
                 cmdlog.cursorRowIndex += 1
                 prog.addProgress(1)
 
-                vd.activeSheet.ensureLoaded()
+                if vd.activeSheet:
+                    vd.activeSheet.ensureLoaded()
                 vd.sync()
                 while not vd.delay():
                     pass
@@ -362,6 +388,9 @@ def replay_sync(vd, cmdlog, live=False):
 @asyncthread
 def replay(vd, cmdlog):
         'Inject commands into live execution with interface.'
+        for thread in vd.threads:
+            if thread.name == 'replay':
+                thread.noblock = True
         vd.replay_sync(cmdlog, live=True)
 
 
@@ -393,16 +422,18 @@ def cmdlog(sheet):
     rows = sheet.cmdlog_sheet.rows
     if isinstance(sheet.source, BaseSheet):
         rows = sheet.source.cmdlog.rows + rows
-    return CommandLog(sheet.name+'_cmdlog', source=sheet, rows=rows)
+    return CommandLogJsonl(sheet.name+'_cmdlog', source=sheet, rows=rows)
 
 
 @BaseSheet.lazy_property
 def cmdlog_sheet(sheet):
-    return CommandLog(sheet.name+'_cmdlog', source=sheet, rows=[])
+    return CommandLogJsonl(sheet.name+'_cmdlog', source=sheet, rows=[])
 
 
 @BaseSheet.property
 def shortcut(self):
+    if self._shortcut:
+        return self._shortcut
     try:
         return str(vd.allSheets.index(self)+1)
     except ValueError:
@@ -416,35 +447,57 @@ def shortcut(self):
     return ''
 
 
-@VisiData.lazy_property
+@VisiData.property
 def cmdlog(vd):
-    vs = CommandLog('cmdlog', rows=[])
-    vd.beforeExecHooks.append(vs.beforeExecHook)
-    return vs
+    if not vd._cmdlog:
+        vd._cmdlog = CommandLogJsonl('cmdlog', rows=[])  # no reload
+        vd._cmdlog.reloadCols()
+        vd.beforeExecHooks.append(vd._cmdlog.beforeExecHook)
+    return vd._cmdlog
 
 @VisiData.property
 def modifyCommand(vd):
-    if vd.activeCommand is not None and isLoggableCommand(vd.activeCommand.longname):
+    if vd.activeCommand is not None and vd.isLoggableCommand(vd.activeCommand.longname):
         return vd.activeCommand
     if not vd.cmdlog.rows:
         return None
     return vd.cmdlog.rows[-1]
 
 
+@CommandLogJsonl.api
+@asyncthread
+def repeat_for_n(cmdlog, r, n=1):
+    r.sheet = r.row = r.col = ""
+    for i in range(n):
+        vd.replayOne(r)
+
+@CommandLogJsonl.api
+@asyncthread
+def repeat_for_selected(cmdlog, r):
+    r.sheet = r.row = r.col = ""
+
+    for idx, r in enumerate(vd.sheet.rows):
+        if vd.sheet.isSelected(r):
+            vd.sheet.cursorRowIndex = idx
+            vd.replayOne(r)
+
+
+BaseSheet.init('_shortcut')
+
 
 globalCommand('gD', 'cmdlog-all', 'vd.push(vd.cmdlog)', 'open global CommandLog for all commands executed in current session')
 globalCommand('D', 'cmdlog-sheet', 'vd.push(sheet.cmdlog)', "open current sheet's CommandLog with all other loose ends removed; includes commands from parent sheets")
-globalCommand('zD', 'cmdlog-sheet-only', 'vd.push(sheet.cmdlog_sheet)', 'open current sheet\'s CommandLog with parent sheets commands\' removed')
+globalCommand('zD', 'cmdlog-sheet-only', 'vd.push(sheet.cmdlog_sheet)', 'open CommandLog for current sheet with commands from parent sheets removed')
 globalCommand('^D', 'save-cmdlog', 'saveSheets(inputPath("save cmdlog to: ", value=fnSuffix(name)), vd.cmdlog, confirm_overwrite=options.confirm_overwrite)', 'save CommandLog to filename.vd file')
 globalCommand('^U', 'replay-pause', 'vd.replay_pause()', 'pause/resume replay')
 globalCommand('^N', 'replay-advance', 'vd.replay_advance()', 'execute next row in replaying sheet')
 globalCommand('^K', 'replay-stop', 'vd.replay_cancel()', 'cancel current replay')
 
 globalCommand(None, 'show-status', 'status(input("status: "))', 'show given message on status line')
-globalCommand('^V', 'show-version', 'status(__version_info__);', 'show version and copyright information on status line')
+globalCommand('^V', 'show-version', 'status(__version_info__);', 'Show version and copyright information on status line')
 globalCommand('z^V', 'check-version', 'checkVersion(input("require version: ", value=__version_info__))', 'check VisiData version against given version')
 
-globalCommand(' ', 'exec-longname', 'execCommand(inputLongname(sheet))', 'execute command by its longname')
+globalCommand(' ', 'exec-longname', 'exec_longname(inputLongname())', 'execute command by its longname')
 
 CommandLog.addCommand('x', 'replay-row', 'vd.replayOne(cursorRow); status("replayed one row")', 'replay command in current row')
 CommandLog.addCommand('gx', 'replay-all', 'vd.replay(sheet)', 'replay contents of entire CommandLog')
@@ -454,8 +507,8 @@ CommandLogJsonl.addCommand('x', 'replay-row', 'vd.replayOne(cursorRow); status("
 CommandLogJsonl.addCommand('gx', 'replay-all', 'vd.replay(sheet)', 'replay contents of entire CommandLog')
 CommandLogJsonl.addCommand('^C', 'replay-stop', 'sheet.cursorRowIndex = sheet.nRows', 'abort replay')
 
-BaseSheet.addCommand('', 'repeat-last', 'execCommand(cmdlog_sheet.rows[-1].longname)', 'run most recent command with an empty, queried input')
-BaseSheet.addCommand('', 'repeat-input', 'r = copy(cmdlog_sheet.rows[-1]); r.sheet=r.row=r.col=""; vd.replayOne(r)', 'run previous command, along with any previous input to that command')
+CommandLog.options.json_sort_keys = False
+CommandLog.options.encoding = 'utf-8'
+CommandLogJsonl.options.json_sort_keys = False
 
-CommandLog.class_options.json_sort_keys = False
-CommandLogJsonl.class_options.json_sort_keys = False
+vd.addGlobals({"CommandLogBase": CommandLogBase})
