@@ -5,9 +5,10 @@ import itertools
 import threading
 import re
 import time
+import json
 
-from visidata import option, options, anytype, stacktrace, vd
-from visidata import asyncthread, dispwidth
+from visidata import options, anytype, stacktrace, vd
+from visidata import asyncthread, dispwidth, clipstr, iterchars
 from visidata import wrapply, TypedWrapper, TypedExceptionWrapper
 from visidata import Extensible, AttrDict, undoAttrFunc
 
@@ -18,13 +19,11 @@ class InProgress(Exception):
 
 INPROGRESS = TypedExceptionWrapper(None, exception=InProgress())  # sentinel
 
-option('col_cache_size', 0, 'max number of cache entries in each cached column')
-option('clean_names', False, 'clean column/sheet names to be valid Python identifiers', replay=True)
+vd.option('col_cache_size', 0, 'max number of cache entries in each cached column')
+vd.option('clean_names', False, 'clean column/sheet names to be valid Python identifiers', replay=True)
 
 __all__ = [
     'clean_to_id',
-    'clean_name',
-    'maybe_clean',
     'Column',
     'setitem',
     'getattrdeep',
@@ -56,16 +55,6 @@ class DisplayWrapper:
         return self.value == other
 
 
-def maybe_clean(s, vs):
-    if (vs or vd).options.clean_names:
-        s = clean_name(s)
-    return s
-
-def clean_name(s):
-    s = re.sub(r'[^\w\d_]', '_', s)  # replace non-alphanum chars with _
-    s = re.sub(r'_+', '_', s)  # replace runs of _ with a single _
-    return s
-
 def clean_to_id(s):  # [Nas Banov] https://stackoverflow.com/a/3305731
     return re.sub(r'\W|^(?=\d)', '_', str(s)).strip('_')
 
@@ -113,6 +102,8 @@ class Column(Extensible):
         self.height = 1       # max height, None/0 to auto-compute for each row
         self.keycol = 0       # keycol index (or 0 if not key column)
         self.expr = None      # Column-type-dependent parameter
+        self.formatter = ''
+        self.defer = False
 
         self.setCache(cache)
         for k, v in kwargs.items():
@@ -131,7 +122,7 @@ class Column(Extensible):
         return self.__copy__()  # no separate deepcopy
 
     def __getstate__(self):
-        d = {k:getattr(self, k) for k in 'name width height expr keycol fmtstr voffset hoffset aggstr'.split()}
+        d = {k:getattr(self, k) for k in 'name width height expr keycol formatter fmtstr voffset hoffset aggstr'.split() if hasattr(self, k)}
         d['type'] = self.type.__name__
         return d
 
@@ -161,7 +152,10 @@ class Column(Extensible):
         else:
             name = str(name)
 
-        self._name = maybe_clean(name, self.sheet)
+        if self.sheet:
+            name = self.sheet.maybeClean(name)
+
+        self._name = name
 
     @property
     def typestr(self):
@@ -200,6 +194,12 @@ class Column(Extensible):
                 vd.addUndo(setattr, self, '_width', self.width)
             self._width = w
 
+    @property
+    def _formatdict(col):
+        if '=' in col.fmtstr:
+            return dict(val.split('=', maxsplit=1) for val in col.fmtstr.split())
+        return {}
+
     @property
     def fmtstr(self):
         'Format string to use to display this column.'
@@ -209,20 +209,48 @@ class Column(Extensible):
     def fmtstr(self, v):
         self._fmtstr = v
 
-    def formatValue(self, typedval):
+    def _format_len(self, typedval, **kwargs):
+        if isinstance(typedval, dict):
+            return f'{{{len(typedval)}}}'
+        elif isinstance(typedval, (list, tuple)):
+            return f'[{len(typedval)}]'
+
+        return self.formatValue(typedval, **kwargs)
+
+    def formatter_len(self, fmtstr):
+        return self._format_len
+
+    def formatter_generic(self, fmtstr):
+        return self.formatValue
+
+    def formatter_json(self, fmtstr):
+        return lambda v,*args,**kwargs: json.dumps(v)
+
+    def formatter_python(self, fmtstr):
+        return lambda v,*args,**kwargs: str(v)
+
+    def make_formatter(self):
+        'Return function for format(v) from the current formatter and fmtstr'
+        _formatMaker = getattr(self, 'formatter_'+(self.formatter or self.sheet.options.disp_formatter))
+        return _formatMaker(self._formatdict)
+
+    def format(self, *args, **kwargs):
+        return self.make_formatter()(*args, **kwargs)
+
+    def formatValue(self, typedval, width=None):
         'Return displayable string of *typedval* according to ``Column.fmtstr``.'
         if typedval is None:
             return None
 
-        if isinstance(typedval, (list, tuple)):
-            return '[%s]' % len(typedval)
-        if isinstance(typedval, dict):
-            return '{%s}' % len(typedval)
+        if self.type is anytype:
+            if isinstance(typedval, (dict, list, tuple)):
+                dispval, dispw = clipstr(iterchars(typedval), width)
+                return dispval
+
         if isinstance(typedval, bytes):
             typedval = typedval.decode(options.encoding, options.encoding_errors)
 
         return vd.getType(self.type).formatter(self.fmtstr, typedval)
-    format=formatValue
 
     def hide(self, hide=True):
         if hide:
@@ -269,7 +297,7 @@ class Column(Extensible):
     def getValue(self, row):
         'Return value for *row* in this column, calculating if not cached.'
 
-        if self.sheet.defer:
+        if self.defer:
             try:
                 row, rowmods = self.sheet._deferredMods[self.sheet.rowid(row)]
                 return rowmods[self]
@@ -334,7 +362,7 @@ class Column(Extensible):
         dw = DisplayWrapper(cellval)
 
         try:
-            dw.display = self.format(typedval) or ''
+            dw.display = self.format(typedval, width=(self.width or 0)*2) or ''
 
             # annotate cells with raw value type in anytype columns, except for strings
             if self.type is anytype and type(cellval) is not str:
@@ -360,15 +388,16 @@ class Column(Extensible):
         return self.getCell(row).display
 
     def putValue(self, row, val):
-        'Change value for *row* in this column to *val* immediately.  Does not check the type.  Overrideable; by default calls ``.setter(row, val)``.'
+        'Change value for *row* in this column to *val* immediately.  Does not check the type.  Overridable; by default calls ``.setter(row, val)``.'
         return self.setter(self, row, val)
 
     def setValue(self, row, val):
-        'Change value for *row* in this column to *val*.  Call ``putValue`` immediately if parent ``sheet.defer`` is False, otherwise cache until later ``putChanges``.  Caller must add undo function.'
-        if self.sheet.defer:
+        'Change value for *row* in this column to *val*.  Call ``putValue`` immediately if not a deferred column (added to deferred parent at load-time); otherwise cache until later ``putChanges``.  Caller must add undo function.'
+        if self.defer:
             self.cellChanged(row, val)
         else:
             self.putValue(row, val)
+        self.sheet.setModified()
 
     def setValueSafe(self, row, value):
         'setValue and ignore exceptions.'
@@ -397,11 +426,11 @@ class Column(Extensible):
         return vd.status('set %d cells to %d values' % (len(rows), len(values)))
 
     def getMaxWidth(self, rows):
-        'Return the maximum length of any cell in column or its header.'
+        'Return the maximum length of any cell in column or its header (up to window width).'
         w = 0
         nlen = dispwidth(self.name)
         if len(rows) > 0:
-            w = max(max(dispwidth(self.getDisplayValue(r)) for r in rows), nlen)+2
+            w = max(max(dispwidth(self.getDisplayValue(r), maxwidth=self.sheet.windowWidth) for r in rows), nlen)+2
         return max(w, nlen)
 
 
@@ -411,27 +440,55 @@ def setitem(r, i, v):  # function needed for use in lambda
     r[i] = v
     return True
 
-def getattrdeep(obj, attr, *default):
-    'Return dotted attr (like "a.b.c") from obj, or default if any of the components are missing.'
-    attrs = attr.split('.')
-    if default:
-        getattr_default = lambda o,a,d=default[0]: getattr(o, a, d)
-    else:
-        getattr_default = lambda o,a: getattr(o, a)
 
-    for a in attrs[:-1]:
-        obj = getattr_default(obj, a)
+def getattrdeep(obj, attr, *default, getter=getattr):
+    try:
+        'Return dotted attr (like "a.b.c") from obj, or default if any of the components are missing.'
+        if not isinstance(attr, str):
+            return getter(obj, attr, *default)
 
-    return getattr_default(obj, attrs[-1])
+        try:  # if attribute exists, return toplevel value, even if dotted
+            if attr in obj:
+                return getter(obj, attr)
+        except Exception as e:
+            pass
 
-def setattrdeep(obj, attr, val):
+        attrs = attr.split('.')
+        for a in attrs[:-1]:
+            obj = getter(obj, a)
+
+        return getter(obj, attrs[-1])
+    except Exception as e:
+        if not default: raise
+        return default[0]
+
+
+def setattrdeep(obj, attr, val, getter=getattr, setter=setattr):
     'Set dotted attr (like "a.b.c") on obj to val.'
+    if not isinstance(attr, str):
+        return setter(obj, attr, val)
+
+    try:  # if attribute exists, overwrite toplevel value, even if dotted
+        getter(obj, attr)
+        return setter(obj, attr, val)
+    except Exception as e:
+        pass
+
     attrs = attr.split('.')
-
     for a in attrs[:-1]:
-        obj = getattr(obj, a)
-    setattr(obj, attrs[-1], val)
+        try:
+            obj = getter(obj, a)
+        except Exception as e:
+            obj = obj[a] = type(obj)()  # assume homogeneous nesting
 
+    setter(obj, attrs[-1], val)
+
+
+def getitemdeep(obj, k, *default):
+    return getattrdeep(obj, k, *default, getter=getitem)
+
+def setitemdeep(obj, k, val):
+    return setattrdeep(obj, k, val, getter=getitemdef, setter=setitem)
 
 def AttrColumn(name='', attr=None, **kwargs):
     'Column using getattr/setattr with *attr*.'
@@ -441,20 +498,25 @@ def AttrColumn(name='', attr=None, **kwargs):
                   setter=lambda col,row,val: setattrdeep(row, col.expr, val),
                   **kwargs)
 
+def getitem(o, k, default=None):
+    return default if o is None else o[k]
+
 def getitemdef(o, k, default=None):
     try:
         return default if o is None else o[k]
     except Exception:
         return default
 
-def ItemColumn(name=None, key=None, **kwargs):
+class ItemColumn(Column):
     'Column using getitem/setitem with *key*.'
-    return Column(name,
-            expr=key if key is not None else name,
-            getter=lambda col,row: getitemdef(row, col.expr),
-            setter=lambda col,row,val: setitem(row, col.expr, val),
+    def __init__(self, name=None, expr=None, **kwargs):
+        super().__init__(name,
+            expr=expr if expr is not None else name,
+            getter=lambda col,row: getitemdeep(row, col.expr, None),
+            setter=lambda col,row,val: setitemdeep(row, col.expr, val),
             **kwargs)
 
+
 class SubColumnFunc(Column):
     'Column compositor; preprocess row with *subfunc*(row, *expr*) before passing to *origcol*.getValue and *origcol*.setValue.'
     def __init__(self, name='', origcol=None, expr=None, subfunc=getitemdef, **kwargs):
@@ -525,10 +587,6 @@ class ExprColumn(Column):
 
 class SettableColumn(Column):
     'Column using rowid to store and retrieve values internally.'
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        self._store = {}
-
     def putValue(self, row, value):
         self._store[self.sheet.rowid(row)] = value
 
@@ -536,6 +594,9 @@ class SettableColumn(Column):
         return self._store.get(self.sheet.rowid(row), None)
 
 
+SettableColumn.init('_store', dict, copy=True)
+
+
 # synonyms
 ColumnItem = ItemColumn
 ColumnAttr = AttrColumn
