@@ -1,10 +1,12 @@
 
-from visidata import *
+from visidata import VisiData, vd, Sheet, Column, Progress, SequenceSheet
 
-option('fixed_rows', 1000, 'number of rows to check for fixed width columns')
-option('fixed_maxcols', 0, 'max number of fixed-width columns to create (0 is no max)')
 
-def open_fixed(p):
+vd.option('fixed_rows', 1000, 'number of rows to check for fixed width columns')
+vd.option('fixed_maxcols', 0, 'max number of fixed-width columns to create (0 is no max)')
+
+@VisiData.api
+def open_fixed(vd, p):
     return FixedWidthColumnsSheet(p.name, source=p, headerlines=[])
 
 class FixedWidthColumn(Column):
@@ -40,7 +42,7 @@ def columnize(rows):
             colstart = i
         prev = i
 
-    yield colstart, None   # final column gets rest of line
+    yield colstart, prev+1   # final column gets rest of line
 
 
 class FixedWidthColumnsSheet(SequenceSheet):
@@ -74,21 +76,23 @@ class FixedWidthColumnsSheet(SequenceSheet):
 
 @VisiData.api
 def save_fixed(vd, p, *vsheets):
-    with p.open_text(mode='w') as fp:
+    with p.open_text(mode='w', encoding=vsheets[0].options.encoding) as fp:
         for sheet in vsheets:
             if len(vsheets) > 1:
-                fp.write('%s\n\n' % vs.name)
+                fp.write('%s\n\n' % sheet.name)
 
+            widths = {}  # Column -> width:int
             # headers
-            for col in sheet.visibleCols:
-                fp.write('{0:{width}}'.format(col.name, width=col.width))
+            for col in Progress(sheet.visibleCols, gerund='sizing'):
+                widths[col] = col.width or sheet.options.default_width or col.getMaxWidth(sheet.rows)
+                fp.write(('{0:%s} ' % widths[col]).format(col.name))
             fp.write('\n')
 
             # rows
             with Progress(gerund='saving'):
                 for dispvals in sheet.iterdispvals(format=True):
                     for col, val in dispvals.items():
-                        fp.write('{0:{align}{width}}'.format(val, width=col.width, align='>' if vd.isNumeric(col) else '<'))
+                        fp.write(('{0:%s%s.%s} ' % ('>' if vd.isNumeric(col) else '<', widths[col], widths[col])).format(val))
                     fp.write('\n')
 
             vd.status('%s save finished' % p)
