@@ -1,8 +1,10 @@
-from visidata import *
+import random
 
-__all__ = ['openurl_postgres', 'openurl_rds', 'PgTable', 'PgTablesSheet']
+from visidata import VisiData, vd, Sheet, options, anytype, urlparse, asyncthread, ColumnItem
 
-option('postgres_schema', 'public', 'The desired schema for the Postgres database')
+__all__ = ['openurl_postgres', 'openurl_postgresql', 'openurl_rds', 'PgTable', 'PgTablesSheet']
+
+vd.option('postgres_schema', 'public', 'The desired schema for the Postgres database')
 
 def codeToType(type_code, colname):
     import psycopg2
@@ -17,7 +19,8 @@ def codeToType(type_code, colname):
     return anytype
 
 
-def openurl_rds(url, filetype=None):
+@VisiData.api
+def openurl_rds(vd, url, filetype=None):
     import boto3
     import psycopg2
 
@@ -37,7 +40,8 @@ def openurl_rds(url, filetype=None):
     return PgTablesSheet(dbname+"_tables", sql=SQL(conn))
 
 
-def openurl_postgres(url, filetype=None):
+@VisiData.api
+def openurl_postgres(vd, url, filetype=None):
     import psycopg2
 
     url = urlparse(url.given)
@@ -52,6 +56,9 @@ def openurl_postgres(url, filetype=None):
     return PgTablesSheet(dbname+"_tables", sql=SQL(conn))
 
 
+VisiData.openurl_postgresql=VisiData.openurl_postgres
+
+
 class SQL:
     def __init__(self, conn):
         self.conn = conn
@@ -70,10 +77,10 @@ class SQL:
             cur.close()
 
 
-def cursorToColumns(cur, sheet):
-    sheet.columns = []
+@VisiData.api
+def postgresGetColumns(vd, cur):
     for i, coldesc in enumerate(cur.description):
-        sheet.addColumn(ColumnItem(coldesc.name, i, type=codeToType(coldesc.type_code, coldesc.name)))
+        yield ColumnItem(coldesc.name, i, type=codeToType(coldesc.type_code, coldesc.name))
 
 
 # rowdef: (table_name, ncols)
@@ -98,7 +105,9 @@ class PgTablesSheet(Sheet):
             r = cur.fetchone()
             if r:
                 self.addRow(r)
-            cursorToColumns(cur, self)
+            self.columns = []
+            for c in vd.postgresGetColumns(cur):
+                self.addColumn(c)
             self.setKeys(self.columns[0:1])  # table_name is the key
 
             for r in cur:
@@ -121,6 +130,8 @@ class PgTable(Sheet):
             r = cur.fetchone()
             if r:
                 self.addRow(r)
-            cursorToColumns(cur, self)
+            self.columns = []
+            for c in vd.postgresGetColumns(cur):
+                self.addColumn(c)
             for r in cur:
                 self.addRow(r)
