@@ -3,24 +3,26 @@ import contextlib
 import itertools
 import collections
 
-from visidata import asyncthread, options, Progress, ColumnItem, SequenceSheet, Sheet, FileExistsError, getType, option, VisiData
+from visidata import vd, asyncthread, options, Progress, ColumnItem, SequenceSheet, Sheet, FileExistsError, getType, VisiData
 from visidata import namedlist, filesize
 
-option('delimiter', '\t', 'field delimiter to use for tsv/usv filetype', replay=True)
-option('row_delimiter', '\n', 'row delimiter to use for tsv/usv filetype', replay=True)
-option('tsv_safe_newline', '\u001e', 'replacement for newline character when saving to tsv', replay=True)
-option('tsv_safe_tab', '\u001f', 'replacement for tab character when saving to tsv', replay=True)
+vd.option('delimiter', '\t', 'field delimiter to use for tsv/usv filetype', replay=True)
+vd.option('row_delimiter', '\n', 'row delimiter to use for tsv/usv filetype', replay=True)
+vd.option('tsv_safe_newline', '\u001e', 'replacement for newline character when saving to tsv', replay=True)
+vd.option('tsv_safe_tab', '\u001f', 'replacement for tab character when saving to tsv', replay=True)
 
 
-def open_tsv(p):
+@VisiData.api
+def open_tsv(vd, p):
     return TsvSheet(p.name, source=p)
 
+
 def splitter(fp, delim='\n'):
     'Generates one line/row/record at a time from fp, separated by delim'
 
     buf = ''
     while True:
-        nextbuf = fp.read(512)
+        nextbuf = fp.read(65536)
         if not nextbuf:
             break
         buf += nextbuf
@@ -33,17 +35,18 @@ def splitter(fp, delim='\n'):
 
 # rowdef: list
 class TsvSheet(SequenceSheet):
-    def iterload(self):
-        delim = self.options.delimiter
-        rowdelim = self.options.row_delimiter
+    delimiter = ''
+    row_delimiter = ''
 
-        with self.source.open_text() as fp:
-            with Progress(total=filesize(self.source)) as prog:
+    def iterload(self):
+        delim = self.delimiter or self.options.delimiter
+        rowdelim = self.row_delimiter or self.options.row_delimiter
+
+        with self.source.open_text(encoding=self.options.encoding) as fp:
                 for line in splitter(fp, rowdelim):
                     if not line:
                         continue
 
-                    prog.addProgress(len(line))
                     row = list(line.split(delim))
 
                     if len(row) < self.nVisibleCols:
@@ -53,20 +56,15 @@ class TsvSheet(SequenceSheet):
                     yield row
 
 
-def load_tsv(fn):
-    vs = open_tsv(Path(fn))
-    yield from vs.iterload()
-
-
 @VisiData.api
-def save_tsv(vd, p, vs):
+def save_tsv(vd, p, vs, delimiter='', row_delimiter=''):
     'Write sheet to file `fn` as TSV.'
-    unitsep = vs.options.delimiter
-    rowsep = vs.options.row_delimiter
+    unitsep = delimiter or vs.options.delimiter
+    rowsep = row_delimiter or vs.options.row_delimiter
     trdict = vs.safe_trdict()
 
-    with p.open_text(mode='w') as fp:
-        colhdr = unitsep.join(col.name.translate(trdict) for col in vs.visibleCols) + options.row_delimiter
+    with p.open_text(mode='w', encoding=vs.options.encoding) as fp:
+        colhdr = unitsep.join(col.name.translate(trdict) for col in vs.visibleCols) + rowsep
         fp.write(colhdr)
 
         for dispvals in vs.iterdispvals(format=True):
@@ -76,6 +74,7 @@ def save_tsv(vd, p, vs):
     vd.status('%s save finished' % p)
 
 
+@Sheet.api
 def append_tsv_row(vs, row):
     'Append `row` to vs.source, creating file with correct headers if necessary. For internal use only.'
     if not vs.source.exists():
@@ -88,10 +87,15 @@ def append_tsv_row(vs, row):
         trdict = vs.safe_trdict()
         unitsep = options.delimiter
 
-        with vs.source.open_text(mode='w') as fp:
-            colhdr = unitsep.join(col.name.translate(trdict) for col in vs.visibleCols) + options.row_delimiter
+        with vs.source.open_text(mode='w', encoding=vs.options.encoding) as fp:
+            colhdr = unitsep.join(col.name.translate(trdict) for col in vs.visibleCols) + vs.options.row_delimiter
             if colhdr.strip():  # is anything but whitespace
                 fp.write(colhdr)
 
-    with vs.source.open_text(mode='a') as fp:
+    with vs.source.open_text(mode='a', encoding=vs.options.encoding) as fp:
         fp.write('\t'.join(col.getDisplayValue(row) for col in vs.visibleCols) + '\n')
+
+
+vd.addGlobals({
+    'TsvSheet': TsvSheet,
+})
