@@ -1,10 +1,13 @@
-from visidata import *
+from visidata import VisiData, vd, Sheet, options, Column, Progress, setitem, ColumnAttr, vlen, RowColorizer, Path, copy
+
+vd.option('xml_parser_huge_tree', True, 'allow very deep trees and very long text content')
 
 
-def open_xml(p):
+@VisiData.api
+def open_xml(vd, p):
     return XmlSheet(p.name, source=p)
 
-open_svg = open_xml
+VisiData.open_svg = VisiData.open_xml
 
 def unns(k):
     'de-namespace key k'
@@ -45,7 +48,8 @@ class XmlSheet(Sheet):
     def iterload(self):
         if isinstance(self.source, Path):
             from lxml import etree, objectify
-            self.root = etree.parse(self.source.open_text())
+            p = etree.XMLParser(**self.options.getall('xml_parser_'))
+            self.root = etree.parse(self.source.open_text(encoding=self.options.encoding), parser=p)
             objectify.deannotate(self.root, cleanup_namespaces=True)
         else: #        elif isinstance(self.source, XmlElement):
             self.root = self.source
@@ -77,11 +81,9 @@ class XmlSheet(Sheet):
 
 @VisiData.api
 def save_xml(vd, p, vs):
-    isinstance(XmlSheet) or vd.fail('must save xml from XmlSheet')
+    isinstance(vs, XmlSheet) or vd.fail('must save xml from XmlSheet')
     vs.root.write(str(p), encoding=options.encoding, standalone=False, pretty_print=True)
 
-VisiData.save_svg = VisiData.save_xml
-
 
 XmlSheet.addCommand('za', 'addcol-xmlattr', 'attr=input("add attribute: "); addColumnAtCursor(AttribColumn(attr, attr))', 'add column for xml attribute')
 XmlSheet.addCommand('v', 'visibility', 'showColumnsBasedOnRow(cursorRow)', 'show only columns in current row attributes')
