@@ -1,11 +1,13 @@
-from visidata import *
 from itertools import chain
 
+from visidata import VisiData, Progress, JsonSheet
 
-def open_yml(p):
+
+@VisiData.api
+def open_yml(vd, p):
     return YamlSheet(p.name, source=p)
 
-open_yaml = open_yml
+VisiData.open_yaml = VisiData.open_yml
 
 class YamlSheet(JsonSheet):
     def iterload(self):
@@ -14,7 +16,7 @@ class YamlSheet(JsonSheet):
             documents = yaml.safe_load_all(fp)
 
             self.columns = []
-            self.colnames = {}
+            self._knownKeys.clear()
 
             # Peek at the document stream to determine how to best DWIM.
             #
