@@ -29,12 +29,32 @@ def formatRange(col, numeric_key):
         return col.format(a)
     return ' - '.join(col.format(x) for x in numeric_key)
 
+
 class RangeColumn(Column):
-    def format(self, typedval):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.formatter = 'range'
+
+    def formatter_range(self, fmtstr):
+        return self._format
+
+    def _format(self, typedval, *args, **kwargs):
         if typedval is None:
             return None
         return formatRange(self.origcol, typedval)
 
+
+def AggrColumn(aggcol, aggregator):
+    aggname = '%s_%s' % (aggcol.name, aggregator.name)
+
+    return Column(aggname,
+                  type=aggregator.type or aggcol.type,
+                  fmtstr=aggcol.fmtstr,
+                  getter=lambda col,row,agg=aggregator: agg(col.origCol, row.sourcerows),
+                  origCol=aggcol,
+                  )
+
+
 class PivotSheet(Sheet):
     'Summarize key columns in pivot table and display as new sheet.'
     rowtype = 'grouped rows'  # rowdef: PivotGroupRow
@@ -71,7 +91,7 @@ class PivotSheet(Sheet):
     def openRow(self, row):
         'open sheet of source rows aggregated in current pivot row'
         vs = copy(self.source)
-        vs.name += "_%s"%"+".join(row.discrete_keys)
+        vs.name += "_%s"%"+".join(map(str, row.discrete_keys))
         vs.rows = sum(row.pivotrows.values(), [])
         return vs
 
@@ -109,11 +129,7 @@ class PivotSheet(Sheet):
         if not self.pivotCols:
             for aggcol, aggregatorlist in aggcols.items():
                 for aggregator in aggregatorlist:
-                    aggname = '%s_%s' % (aggcol.name, aggregator.name)
-
-                    c = Column(aggname,
-                                type=aggregator.type or aggcol.type,
-                                getter=lambda col,row,aggcol=aggcol,agg=aggregator: agg(aggcol, row.sourcerows))
+                    c = AggrColumn(aggcol, aggregator)
                     self.addColumn(c)
 
         # add pivoted columns
@@ -172,7 +188,7 @@ class PivotSheet(Sheet):
         numericBins = []
         degenerateBinning = False
         if numericCols:
-            nbins = options.histogram_bins or int(len(self.source.rows) ** (1./2))
+            nbins = self.source.options.histogram_bins or int(len(self.source.rows) ** (1./2))
             vals = tuple(numericCols[0].getValues(self.source.rows))
             minval = min(vals)
             maxval = max(vals)
@@ -250,4 +266,19 @@ class PivotSheet(Sheet):
             c.setCache(True)
 
 
+@PivotSheet.api
+def addcol_aggr(sheet, col):
+    hasattr(col, 'origCol') or vd.fail('not an aggregation column')
+    for agg in vd.chooseMany(vd.aggregator_choices):
+        sheet.addColumnAtCursor(AggrColumn(col.origCol, vd.aggregators[agg]))
+
+
 Sheet.addCommand('W', 'pivot', 'vd.push(Pivot(sheet, keyCols, [cursorCol]))', 'open Pivot Table: group rows by key column and summarize current column')
+
+PivotSheet.addCommand('', 'addcol-aggr', 'addcol_aggr(cursorCol)', 'add aggregation column from source of current column')
+vd.addMenuItem('Column', 'Add column', 'aggregator', 'addcol-aggr')
+
+vd.addGlobals({
+    'Pivot': Pivot,
+    'PivotGroupRow': PivotGroupRow,
+})
