@@ -1,38 +1,84 @@
 from functools import singledispatch
+from typing import Mapping
+import inspect
+import math
 
 from visidata import *
 
-__all__ = ['PythonSheet', 'expand_cols_deep', 'deduceType', 'closeColumn', 'ListOfDictSheet', 'SheetDict', 'PyobjSheet', 'view']
-
-option('visibility', 0, 'visibility level')
-option('expand_col_scanrows', 1000, 'number of rows to check when expanding columns (0 = all)')
+vd.option('visibility', 0, 'visibility level')
+vd.option('default_sample_size', 100, 'number of rows to sample for regex.split (0=all)', replay=True)
+vd.option('fmt_expand_dict', '%s.%s', 'format str to use for names of columns expanded from dict (colname, key)') #1457
+vd.option('fmt_expand_list', '%s[%s]', 'format str to use for names of columns expanded from list (colname, index)')
 
 
 class PythonSheet(Sheet):
     def openRow(self, row):
         return PyobjSheet("%s[%s]" % (self.name, self.keystr(row)), source=row)
 
-def _getWraparoundSlice(seq, n, center):
-    '''Return a slice of length n from a sequence, centered around a given index'''
-    start = int(center - n / 2) % len(seq)
+
+class InferColumnsSheet(Sheet):
+    _rowtype = dict
+    @asyncthread
+    def reload(self):
+        self.reloadCols()
+
+        self.rows = []
+        for r in self.iterload():
+            self.addRow(r)
+
+        # if an ordering has been specified, sort the sheet
+        if self._ordering:
+            vd.sync(self.sort())
+
+    def reloadCols(self):
+        self.columns = []
+        self._knownKeys.clear()
+        for c in type(self).columns:
+            self.addColumn(deepcopy(c))
+
+    def addColumn(self, *cols, index=None):
+        for c in cols:
+            self._knownKeys.add(c.expr or c.name)
+        return super().addColumn(*cols, index=index)
+
+    def addRow(self, row, index=None):
+        ret = super().addRow(row, index=index)
+        for k in row:
+            if k not in self._knownKeys:
+                self.addColumn(ColumnItem(k, type=deduceType(row[k])))
+
+        return ret
+
+
+InferColumnsSheet.init('_knownKeys', set, copy=True)  # set of row keys already seen
+InferColumnsSheet.init('_ordering', list, copy=True)
+
+
+@Sheet.api
+def getSampleRows(sheet):
+    'Return list of sample rows centered around the cursor.'
+    n = sheet.options.default_sample_size
+    if n == 0 or n >= sheet.nRows:
+        return sheet.rows
+
+    vd.warning(f'sampling {n} rows')
+    seq = sheet.rows
+    start = math.ceil(sheet.cursorRowIndex - n / 2) % len(seq)
     end = (start + n) % len(seq)
     if start < end:
         return seq[start:end]
     return seq[start:] + seq[:end]
 
+
 @asyncthread
 def expand_cols_deep(sheet, cols, rows=None, depth=0):  # depth == 0 means drill all the way
     'expand all visible columns of containers to the given depth (0=fully)'
     ret = []
     if not rows:
-        scanrows = options.expand_col_scanrows
-        if scanrows == 0 or scanrows >= sheet.nRows:
-            rows = sheet.rows
-        else:
-            rows = _getWraparoundSlice(sheet.rows, scanrows, sheet.cursorRowIndex)
+        rows = sheet.getSampleRows()
 
     for col in cols:
-        newcols = _addExpandedColumns(col, rows, sheet.columns.index(col))
+        newcols = col.expand(rows)
         if depth != 1:  # countdown not yet complete, or negative (indefinite)
             ret.extend(expand_cols_deep.__wrapped__(sheet, newcols, rows, depth-1))
     return ret
@@ -58,23 +104,40 @@ def _(sampleValue, col, vals):
         })
 
     return [
-        ExpandedColumn('%s.%s' % (col.name, k), type=v, origCol=col, key=k)
+        ExpandedColumn(col.sheet.options.fmt_expand_dict % (col.name, k), type=v, origCol=col, expr=k)
             for k, v in newcols.items()
     ]
 
+def _createExpandedColumnsNamedTuple(col, val):
+    return [
+        ExpandedColumn(col.sheet.options.fmt_expand_dict % (col.name, k), type=colType, origCol=col, expr=i)
+            for i, (k, colType) in enumerate(zip(val._fields, (deduceType(v) for v in val)))
+    ]
+
 @_createExpandedColumns.register(list)
 @_createExpandedColumns.register(tuple)
 def _(sampleValue, col, vals):
     '''Use the longest sequence to determine the number of columns we need to
     create, and their presumed types'''
-    longestSeq = max(vals, key=len)
+    def lenNoExceptions(v):
+        try:
+            return len(v)
+        except Exception as e:
+            return 0
+
+    if hasattr(sampleValue, '_fields'):  # looks like a namedtuple
+        return _createExpandedColumnsNamedTuple(col, vals[0])
+
+    longestSeq = max(vals, key=lenNoExceptions)
     colTypes = [deduceType(v) for v in longestSeq]
     return [
-        ExpandedColumn('%s[%s]' % (col.name, k), type=colType, origCol=col, key=k)
+        ExpandedColumn(col.sheet.options.fmt_expand_list % (col.name, k), type=colType, origCol=col, expr=k)
             for k, colType in enumerate(colTypes)
     ]
 
-def _addExpandedColumns(col, rows, idx):
+
+@Column.api
+def expand(col, rows):
     isNull = col.sheet.isNullFunc()
     nonNulls = [
         col.getTypedValue(row)
@@ -89,6 +152,8 @@ def _addExpandedColumns(col, rows, idx):
     # column can be expanded.
     expandedCols = _createExpandedColumns(nonNulls[0], col, nonNulls)
 
+    idx = col.sheet.columns.index(col)
+
     for i, c in enumerate(expandedCols):
         col.sheet.addColumn(c, index=idx+i+1)
     if expandedCols:
@@ -105,10 +170,10 @@ def deduceType(v):
 
 class ExpandedColumn(Column):
     def calcValue(self, row):
-        return getitemdef(self.origCol.getValue(row), self.key)
+        return getitemdef(self.origCol.getValue(row), self.expr)
 
     def setValue(self, row, value):
-        self.origCol.getValue(row)[self.key] = value
+        self.origCol.getValue(row)[self.expr] = value
 
 
 def closeColumn(sheet, col):
@@ -123,8 +188,9 @@ def closeColumn(sheet, col):
 
 
 #### generic list/dict/object browsing
-def view(obj):
-    run(PyobjSheet(getattr(obj, '__name__', ''), source=obj))
+@VisiData.global_api
+def view(vd, obj):
+    vd.run(PyobjSheet(getattr(obj, '__name__', ''), source=obj))
 
 
 
@@ -140,12 +206,9 @@ def AttrColumns(attrnames):
     'Return column names for all elements of list `attrnames`.'
     return [ColumnAttr(name) for name in attrnames]
 
-def DictKeyColumns(d):
-    'Return a list of Column objects from dictionary keys.'
-    return [ColumnItem(k, k, type=deduceType(d[k])) for k in d.keys()]
 
 def SheetList(*names, **kwargs):
-    'Creates a Sheet from a list of homogenous dicts or namedtuples.'
+    'Creates a Sheet from a list of homogeneous dicts or namedtuples.'
 
     src = kwargs.get('source', None)
     if not src:
@@ -181,14 +244,12 @@ class ListOfDictSheet(PythonSheet):
     rowtype = 'dicts'
     def reload(self):
         self.columns = []
-        addedCols = set()
+        self._knownKeys = set()
         for row in self.source:
-            newCols = {k: v for k, v in row.items() if k not in addedCols}
-            if not newCols:
-                continue
-            for c in DictKeyColumns(newCols):
-                self.addColumn(c)
-                addedCols.add(c.name)
+            for k in row:
+                if k not in self._knownKeys:
+                    self.addColumn(ColumnItem(k, k, type=deduceType(row[k])))
+                    self._knownKeys.add(k)
         self.rows = self.source
 
 # rowdef: namedtuple
@@ -253,6 +314,7 @@ class PyobjSheet(PythonSheet):
     columns = [
         Column('attribute'),
         ColumnSourceAttr('value'),
+        Column('signature', width=0, getter=lambda c,r: dict(inspect.signature(getattr(c.sheet.source, r)).parameters)),
         Column('docstring', getter=lambda c,r: docstring(c.sheet.source, r))
     ]
     nKeys = 1
@@ -265,7 +327,7 @@ class PyobjSheet(PythonSheet):
                 return SheetNamedTuple(*names, **kwargs)
             else:
                 return SheetList(*names, **kwargs)
-        elif isinstance(pyobj, dict):
+        elif isinstance(pyobj, Mapping):
             return SheetDict(*names, **kwargs)
         elif isinstance(pyobj, str):
             return TextSheet(*names, source=pyobj.splitlines())
@@ -279,7 +341,7 @@ class PyobjSheet(PythonSheet):
 
     def reload(self):
         self.rows = []
-        vislevel = options.visibility
+        vislevel = self.options.visibility
         for r in dir(self.source):
             try:
                 if vislevel <= 2 and r.startswith('__'): continue
@@ -294,12 +356,20 @@ class PyobjSheet(PythonSheet):
         v = getattr(self.source, row)
         return PyobjSheet(self.name + "." + str(row), source=v() if callable(v) else v)
 
+
 @TableSheet.api
 def openRow(sheet, row):
     'Return Sheet diving into *row*.'
-    k = sheet.keystr(row) or [sheet.cursorRowIndex]
+    k = sheet.keystr(row) or sheet.cursorRowIndex
     name = f'{sheet.name}[{k}]'
-    return PyobjSheet(name, source=tuple(c.getTypedValue(row) for c in sheet.visibleCols))
+    return TableSheet(name,
+                      rows=sheet.visibleCols,
+                      sourceRow=sheet.cursorRow,
+                      columns=[
+                        Column('column', getter=lambda c,r: r.name),
+                        Column('value', getter=lambda c,r: r.getTypedValue(c.sheet.sourceRow), setter=lambda c,r,v: r.setValue(c.sheet.sourceRow, v)),
+                      ],
+                      nKeys=1)
 
 @TableSheet.api
 def openCell(sheet, col, row):
@@ -309,22 +379,34 @@ def openCell(sheet, col, row):
     return PyobjSheet(name, source=col.getTypedValue(row))
 
 
-globalCommand('^X', 'pyobj-expr', 'expr = input("eval: ", "expr", completer=CompleteExpr()); vd.push(PyobjSheet(expr, source=evalExpr(expr, None)))', 'evaluate Python expression and open result as Python object')
-globalCommand('g^X', 'exec-python', 'expr = input("exec: ", "expr", completer=CompleteExpr()); exec(expr, getGlobals())', 'execute Python statement in the global scope')
-globalCommand('z^X', 'pyobj-expr-row', 'expr = input("eval over current row: ", "expr", completer=CompleteExpr()); vd.push(PyobjSheet(expr, source=evalExpr(expr, cursorRow)))', 'evaluate Python expression, in context of current row, and open result as Python object')
+@BaseSheet.api
+def pyobj_expr(sheet):
+    def launch_repl(v, i):
+        import code
+        with SuspendCurses():
+            code.InteractiveConsole(locals=locals()).interact()
+        return v, i
+    expr = vd.input("eval: ", "expr", completer=visidata.CompleteExpr(), bindings={'^X': launch_repl})
+    vd.push(PyobjSheet(expr, source=sheet.evalExpr(expr)))
+
+BaseSheet.addCommand('^X', 'pyobj-expr', 'pyobj_expr()', 'evaluate Python expression and open result as Python object')
+BaseSheet.addCommand('', 'exec-python', 'expr = input("exec: ", "expr", completer=CompleteExpr()); exec(expr, getGlobals(), LazyChainMap(sheet, *vd.contexts, locals=vd.getGlobals()))', 'execute Python statement with expression scope')
+BaseSheet.addCommand('g^X', 'import-python', 'modname=input("import: ", type="import_python"); exec("import "+modname, getGlobals())', 'import Python module in the global scope')
+globalCommand('z^X', 'pyobj-expr-row', 'expr = input("eval over current row: ", "expr", completer=CompleteExpr()); vd.push(PyobjSheet(expr, source=evalExpr(expr, row=cursorRow)))', 'evaluate Python expression, in context of current row, and open result as Python object')
 
 Sheet.addCommand('^Y', 'pyobj-row', 'status(type(cursorRow)); vd.push(PyobjSheet("%s[%s]" % (sheet.name, cursorRowIndex), source=cursorRow))', 'open current row as Python object')
 Sheet.addCommand('z^Y', 'pyobj-cell', 'status(type(cursorValue)); vd.push(PyobjSheet("%s[%s].%s" % (sheet.name, cursorRowIndex, cursorCol.name), source=cursorValue))', 'open current cell as Python object')
 globalCommand('g^Y', 'pyobj-sheet', 'status(type(sheet)); vd.push(PyobjSheet(sheet.name+"_sheet", source=sheet))', 'open current sheet as Python object')
 
-Sheet.addCommand('(', 'expand-col', 'expand_cols_deep(sheet, [cursorCol], depth=0)', 'expand current column of containers fully')
-Sheet.addCommand('g(', 'expand-cols', 'expand_cols_deep(sheet, visibleCols, depth=0)', 'expand all visible columns of containers fully')
+Sheet.addCommand('(', 'expand-col', 'expand_cols_deep(sheet, [cursorCol], depth=1)', 'expand current column of containers one level')
+Sheet.addCommand('g(', 'expand-cols', 'expand_cols_deep(sheet, visibleCols, depth=1)', 'expand all visible columns of containers one level')
 Sheet.addCommand('z(', 'expand-col-depth', 'expand_cols_deep(sheet, [cursorCol], depth=int(input("expand depth=", value=1)))', 'expand current column of containers to given depth (0=fully)')
 Sheet.addCommand('gz(', 'expand-cols-depth', 'expand_cols_deep(sheet, visibleCols, depth=int(input("expand depth=", value=1)))', 'expand all visible columns of containers to given depth (0=fully)')
 
 Sheet.addCommand(')', 'contract-col', 'closeColumn(sheet, cursorCol)', 'unexpand current column; restore original column and remove other columns at this level')
 
-Sheet.addCommand(ENTER, 'open-row', 'vd.push(openRow(cursorRow))', 'open sheet with copies of rows referenced in current row')
+Sheet.addCommand('', 'open-row-basic', 'vd.push(TableSheet.openRow(sheet, cursorRow))', 'open sheet with open sheet with copies of rows referenced in current row')
+Sheet.addCommand(ENTER, 'open-row', 'vd.push(openRow(cursorRow))', 'open current row with sheet-specific dive')
 Sheet.addCommand('z'+ENTER, 'open-cell', 'vd.push(openCell(cursorCol, cursorRow))', 'open sheet with copies of rows referenced in current cell')
 Sheet.addCommand('g'+ENTER, 'dive-selected', 'for r in selectedRows: vd.push(openRow(r))', 'open sheet with copies of rows referenced in selected rows')
 Sheet.addCommand('gz'+ENTER, 'dive-selected-cells', 'for r in selectedRows: vd.push(openCell(cursorCol, r))', 'open sheet with copies of rows referenced in selected rows')
@@ -332,3 +414,16 @@ Sheet.addCommand('gz'+ENTER, 'dive-selected-cells', 'for r in selectedRows: vd.p
 PyobjSheet.addCommand('v', 'visibility', 'sheet.options.visibility = 0 if sheet.options.visibility else 2; reload()', 'toggle show/hide for methods and hidden properties')
 PyobjSheet.addCommand('gv', 'show-hidden', 'sheet.options.visibility = 2; reload()', 'show methods and hidden properties')
 PyobjSheet.addCommand('zv', 'hide-hidden', 'sheet.options.visibility -= 1; reload()', 'hide methods and hidden properties')
+
+vd.addGlobals({
+    'ExpandedColumn': ExpandedColumn,
+    'PythonSheet': PythonSheet,
+    'expand_cols_deep': expand_cols_deep,
+    'deduceType': deduceType,
+    'closeColumn': closeColumn,
+    'ListOfDictSheet': ListOfDictSheet,
+    'SheetDict': SheetDict,
+    'InferColumnsSheet': InferColumnsSheet,
+    'PyobjSheet': PyobjSheet,
+    'view': view,
+})
